@@ -28,6 +28,7 @@
 
 #include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <libgnomeui/gnome-help.h>
 #include <panel-applet.h>
 
 #include "drive-list.h"
@@ -88,7 +89,7 @@ change_background (PanelApplet               *applet,
     gtk_widget_set_style (GTK_WIDGET (applet), NULL);
     rc_style = gtk_rc_style_new ();
     gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
-    g_object_unref (rc_style);
+    gtk_rc_style_unref (rc_style);
 
     switch (type) {
     case PANEL_NO_BACKGROUND:
@@ -103,6 +104,7 @@ change_background (PanelApplet               *applet,
 	    g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
 	style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
 	gtk_widget_set_style (GTK_WIDGET (applet), style);
+	g_object_unref (style);
 	break;
     }
 }
@@ -130,7 +132,7 @@ display_about_dialog (BonoboUIComponent *uic,
 	"authors",     authors,
 	"documenters", documenters,
 	"translator-credits", _("translator_credits"),
-	"logo_icon_name",     "drivemount-applet",
+	"logo_icon_name",     "gnome-dev-jazdisk",
 	NULL);
 }
 
@@ -179,7 +181,7 @@ applet_factory (PanelApplet *applet,
     GtkWidget *drive_list;
 
     if (!strcmp (iid, drivemount_iid)) {
-	gtk_window_set_default_icon_name ("drivemount-applet");
+	gtk_window_set_default_icon_name ("gnome-dev-jazdisk");
 
 	panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
 	panel_applet_add_preferences (applet,
