@@ -11,11 +11,23 @@
 #include <config.h>
 #endif
 
-#include <gnome.h>
+#include <string.h>
+
+#include <gtk/gtkcheckbutton.h>
+#include <gtk/gtkcolorbutton.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtknotebook.h>
+#include <gtk/gtksizegroup.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtkvbox.h>
+
 #include <gconf/gconf-client.h>
+#include <libgnome/gnome-help.h>
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
-#include <string.h>
 
 #include "global.h"
 
@@ -71,7 +83,7 @@ key_writable (PanelApplet *applet, const char *key)
 }
 
 
-void
+static void
 properties_set_insensitive(MultiloadApplet *ma)
 {
 	gint i, total_graphs, last_graph;
